"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.fillInOptions = exports.defaultMultilineArrayOptions = exports.optionHelp = exports.untilWrapThresholdRegExp = exports.elementWrapThreshold = exports.untilLineTriggerRegExp = exports.elementsPerLineTrigger = exports.envDebugKey = void 0;
const web_index_1 = require("augment-vir/dist/web-index");
exports.envDebugKey = 'NEW_LINE_DEBUG';
exports.elementsPerLineTrigger = 'prettier-elements-per-line:';
exports.untilLineTriggerRegExp = new RegExp(`.*${exports.elementsPerLineTrigger}`);
exports.elementWrapThreshold = 'prettier-wrap-threshold:';
exports.untilWrapThresholdRegExp = new RegExp(`.*${exports.elementWrapThreshold}`);
exports.optionHelp = {
    multilineArrayWrapThreshold: `A number indicating that all arrays should wrap when they have MORE than the specified number. Defaults to 0, indicating that all arrays will wrap.\nExample: multilineArrayWrapThreshold: 3,\nCan be overridden with a comment starting with ${exports.elementWrapThreshold}.\nComment example: // ${exports.elementWrapThreshold} 5`,
    multilineArrayElementsPerLine: `A string with a space separated list of numbers indicating how many elements should be on each line. The pattern repeats if an array is longer than the pattern. Defaults to an empty string. Any invalid numbers causes the whole pattern to revert to the default. This overrides the wrap threshold option.\nExample: elementsPerLinePattern: "3 2 1"\nCan be overridden with a comment starting with ${exports.elementsPerLineTrigger}.\nComment example: // ${exports.elementsPerLineTrigger} 3 2 1\nThis option overrides Prettier's default wrapping; multiple elements on one line will not be wrapped even if they don't fit within the column count.`,
};
const optionPropertyValidators = {
    multilineArrayWrapThreshold: (input) => typeof input === 'number' && !isNaN(input),
    multilineArrayElementsPerLine: (input) => typeof input === 'string',
};
exports.defaultMultilineArrayOptions = {
    multilineArrayWrapThreshold: 0,
    multilineArrayElementsPerLine: '',
};
function fillInOptions(input) {
    if (!input || typeof input !== 'object') {
        return exports.defaultMultilineArrayOptions;
    }
    const newOptions = {};
    (0, web_index_1.getObjectTypedKeys)(exports.defaultMultilineArrayOptions).forEach((optionsKey) => {
        const inputValue = input[optionsKey];
        const defaultValue = exports.defaultMultilineArrayOptions[optionsKey];
        if (!optionPropertyValidators[optionsKey](inputValue)) {
            if (inputValue != undefined) {
                // only log the error when the key is actually provided
                console.error(`Invalid type for Prettier options key ${optionsKey}. Expected ${typeof defaultValue} but got ${typeof inputValue}.`);
            }
            newOptions[optionsKey] =
                defaultValue;
        }
        else {
            newOptions[optionsKey] = inputValue;
        }
    });
    return newOptions;
}
exports.fillInOptions = fillInOptions;
